zmq = require 'zmq'
debug = require("debug")("omg-zmqsockets")
z85 = require("z85")
fs = require 'fs'
child_process = require 'child_process'

try
  serverPrivateKey = fs.readFileSync "/data/curvekeys/serverPrivate"
  producerPublicKey = fs.readFileSync "/data/curvekeys/producerPublic"
  clientPublicKey = fs.readFileSync "/data/curvekeys/clientPublic"
catch e
  child_process.execSync "omgcurvekeygen"
  serverPrivateKey = fs.readFileSync("/data/curvekeys/serverPrivate")
  producerPublicKey = fs.readFileSync "/data/curvekeys/producerPublic"
  clientPublicKey = fs.readFileSync "/data/curvekeys/clientPublic"

serverPrivateKey = serverPrivateKey.toString()
producerPublicKey = producerPublicKey.toString()
clientPublicKey = clientPublicKey.toString()


configloader = require 'omg-configloader'

zmqconfig = configloader.zmq
porthandlerConfig = zmqconfig.porthandler

getPorthandlerPort = ->
  transport = porthandlerConfig.transport or zmqconfig.standardTransport
  address = porthandlerConfig.address or zmqconfig.standardAddress
  port = porthandlerConfig.rep
  return "#{transport}://#{address}:#{port}"

config = configloader.producers

producerSockets = {}

 # receive response from porthandler

requestFreePort = (name, socketType, cb) ->
  porthandler = zmq.socket "req"
  porthandler.connect getPorthandlerPort()
  porthandler.once "message", (port) ->
    porthandler.close()
    cb port
  porthandler.send [name, socketType]


socketsBeingCreated = {}

sendSocketWhenReady = (combinedName, cb) ->
  retry = ->
    if producerSockets[combinedName] and producerSockets[combinedName].send
      cb producerSockets[combinedName]
    else
      sendSocketWhenReady combinedName, cb
  setTimeout retry, 5


create = (producerName, socketType, cb) ->
  combinedName = "#{producerName}#{socketType}"
  if producerSockets[combinedName] and producerSockets[combinedName].send
    debug "socket already bound"
    cb producerSockets[combinedName]
    return

  if socketsBeingCreated[combinedName]
    debug "Socket is being bound already"
    sendSocketWhenReady combinedName, cb
    return

  debug "new socket #{combinedName}"
  socketsBeingCreated[combinedName] = true

  requestFreePort producerName, socketType, (port) ->
    transport = config[producerName] and config[producerName].transport or config.standardTransport
    address = config[producerName] and config[producerName].address or config.standardAddress
    portToBind = config[producerName] and config[producerName][socketType] or "#{transport}://#{address}:#{port}"

    debug "binding to port: #{portToBind}"
    socket = zmq.socket socketType
    socket.curve_server = 1
    socket.curve_secretkey = serverPrivateKey

    socket.bind portToBind, (err) ->
      producerSockets[combinedName] = socket
      cb producerSockets[combinedName]

connect = (producerName, socketType, cb) ->
  combinedName = "#{producerName}#{socketType}"
  requestFreePort producerName, socketType, (port) ->
    cb(port.toString())


class ZapHandler
  constructor: ->
    debug "creating new zap handler"

    zmqzap = require("zmq-zap")
    ZAP = zmqzap.ZAP
    CurveMechanism = zmqzap.CurveMechanism

    zap = new ZAP()

    zap.use new CurveMechanism((data, callback) ->

      # This is where you'd check to see if you want to let the socket connect.
      # The CURVE mechanism lets you authenticate based on domain and address and a pre-exchanged publickey.

      # where the publickey matches the one we have for the client
      if data.publickey is producerPublicKey or data.publickey is clientPublicKey
        debug "Auth successful"
        callback null, true
      else
        debug "Auth failed"
        callback null, false
    )

    # Setup ZeroMQ ZAP socket
    # We'll use a router so that we can handle multiple requests at once
    zapSocket = zmq.socket("router")
    zapSocket.on "message", ->

      # When we get a message, send it through to the ZAP handler
      zap.authenticate arguments, (err, response) ->
        console.error "Error:", err  if err

        # Always send the response if the handler gives us one in the callback.
        # This should be done even if there is an error so that we don't block any sockets.
        zapSocket.send response  #if response


    # The socket for the ZAP handler should be bound before creating any sockets that will use it.
    # We'll use bindSync to make sure that the bind completes before we do anything else.
    zapSocket.bindSync "inproc://zeromq.zap.01"

module.exports =
  create: create
  connectPort: connect
  ZapHandler: ZapHandler